/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
   
    file: process.cpp
*/

#include "core.h"




// buffers for image processing
color_type *last_frame = NULL;
color_type *temp_frame = NULL;
color_type *save_frame = NULL;


color_type *back_buffers[NUM_BACK_BUFFERS] = {NULL};


/* Active filters in the pipe line */
bool filter_gauss = false;
bool filter_rc = false;
bool filter_sharpness = false;
bool filter_edgefast = false;
bool filter_laplace = false;
bool filter_sobel = false;
bool filter_emboss = false;
bool filter_threshold = false;
bool filter_tracking = false;
bool filter_erode = false;
bool filter_reduceedge = false;
bool filter_colorcorrect = false;

bool filter_average = false;
bool filter_comboedge = false;
bool filter_canny = false;

bool filter_speed = false;



bool d_frame = false;       // Display difference frame


double threshold_1 = 42;
double threshold_2 = 260;





/* Clear all off screen buffers */
void clear_buffers( void )
{
    memset(camera_buffer, 0, sizeof(color_type) * size_x * size_y);
    memset(temp_frame, 0, sizeof(color_type) * size_x * size_y);
    memset(last_frame, 0, sizeof(color_type) * size_x * size_y);
    memset(save_frame, 0, sizeof(color_type) * size_x * size_y);
}
/* End of clear buffers */


/* Create image processing buffers */
void process_start( void )
{
    if (last_frame != NULL) free(last_frame);    

    last_frame = (color_type *) malloc(camera_x_size * camera_y_size* sizeof(color_type));
        
    if (last_frame == NULL)
    {            
        add_line("Unable to create display buffer");
        return;
    }     
    
    if (temp_frame != NULL) free(temp_frame);    

    temp_frame= (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (temp_frame == NULL)
    {            
        add_line("Unable to create display buffer");
        return;
    }    
    
   save_frame= (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (save_frame == NULL)
    {            
        add_line("Unable to create display buffer");
        return;
    }    
    

for (int i = 0; i < NUM_BACK_BUFFERS; i++)

{
   back_buffers[i] = (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (back_buffers[i] == NULL)
    {            
        add_line("Unable to create back");
        return;
    }    


}















    // Set image size for filter.cpp
    size_x = camera_x_size;
    size_y = camera_y_size;
}
/* End of process start */


/* Destroy image processing buffers */
void process_stop ( void )
{
    if (last_frame) free(last_frame);
    if (temp_frame) free(temp_frame);
    if (save_frame) free(temp_frame);
    
    
    
    for (int i = 0; i < NUM_BACK_BUFFERS; i++)
    {
        if (back_buffers[i])
            free(back_buffers[i]);
    }
  
    
}
/* End of process_stop */


/* Swap two pointers */
void swap(color_type *&a, color_type *&b)
{
    color_type *t;
    
    t = a;
    a = b;
    b = t;
}
/* End of swap */


/* This implementes the filter pipeline */
void process_frame( void )
{
    // S is source, D is destination
    // The swaps later on keep this correct
    // This is to aviod un need data copying

    color_type *s = camera_buffer;
    color_type *d = temp_frame;






    /* Active filters in the pipe line */

    // Save source image
    memcpy(save_frame, s, DATA_SIZE);

    if (filter_gauss)
    {
        transform_frame( s, d, gauss_blur, 5, 1/159.0, 0);
        swap(s, d);
    }
     
    if (filter_rc)
    {
        rc_filter(s, d); 
        swap(s, d);
    }

    if (filter_sharpness)
    {
        transform_frame(s, d, sharpness, 3, 1, 0);
        swap(s, d);
    }

    if (filter_canny)
    {
       canny_edge(s, d, threshold_1, threshold_2, 128);
         swap(s, d);
    }



    if (filter_edgefast)
    {
        edge_detect_frame(s, d);   
        swap(s, d);
    } 
 
    if (filter_laplace)
    {
        transform_frame(s, d, laplace5, 5, 1, 128);        
        swap(s, d);
    } 
  
    if (filter_sobel)
    {
        general_filter_frame(s, d, sobel_x, sobel_y, 3, 1/3.0);
        swap(s, d);
    } 

    if (filter_comboedge)
    {
        combo_edgedetect_frame(s, d, threshold_1);
        swap(s, d);
    }

    if (filter_average)
    {
        average_frame( s, d);
        swap(s, d);
    }

    if (filter_emboss)
    {
        transform_frame(s, d, embossing, 3, 1, 128); 
        swap(s, d);        
    } 

    if (filter_threshold)
    {
        threshold_frame(s, d, threshold_1);
        swap(s, d);
    }
    
    if (filter_erode)
    {
        erode_frame(s, d, threshold_1);         
        swap(s, d);
    }

    if (filter_reduceedge)
    {
        reduceedge_frame(s, d, threshold_2);
        swap(s, d);
    }
        
    if (filter_tracking)
    {
        target_frame(s, d);
         swap(s, d);
    }


    if (filter_speed)
    {
        speed_frame(s, d);
         swap(s, d);
    }

    // Copy back into camera buffer
    memcpy(d, s, camera_x_size * camera_y_size * sizeof(color_type));
    
    
    
    
    
    for (int i = 0; i < NUM_BACK_BUFFERS-1; i++)
    {
        memcpy(back_buffers[i], back_buffers[i+1],   camera_x_size * camera_y_size * sizeof(color_type));
    } 
    
    
    memcpy(back_buffers[NUM_BACK_BUFFERS-1],camera_buffer,   camera_x_size * camera_y_size * sizeof(color_type));
    
  
    
    
    
      // Copy back into camera buffer
    memcpy(d, s, camera_x_size * camera_y_size * sizeof(color_type));
    
    
    
    
    
    
    
}
/* End of process frame */


/* Show active filters */
void show_filters( void )
{
    char b[1000];

    strcpy(b, "Filters: ");
    
    if (filter_gauss)       strcat(b, "gauss ");
    if (filter_rc)          strcat(b, "rc ");
    if (filter_sharpness)   strcat(b, "sharpness ");
    if (filter_canny)       strcat(b, "canny ");
    if (filter_edgefast)    strcat(b, "edgefast ");
    if (filter_laplace)     strcat(b, "laplace "); 
    if (filter_sobel )      strcat(b, "sobel ");
    if (filter_emboss)      strcat(b, "emboss ");
    if (filter_threshold )  strcat(b, "threshold  ");
    if (filter_erode )      strcat(b, "erode  ");
    if (filter_tracking )   strcat(b, "tracking  ");
    if (filter_reduceedge ) strcat(b, "reduceedge ");
    if (filter_average )    strcat(b, "average");
    if (filter_comboedge)    strcat(b, "combo edge");    
    
    add_line(b);
}
/* end of show filters */












