

#include "core.h"
#include "timer.h"



    float speed_limit = 25;

    float min_time = 0.05;
    float dist = 30.2; //  dist in feet
           
    float timeout = 8;
    float clear = 2;
      
    float alpha = 0.80;
    float act_alpha = 0.10;  
     
    float fact = 2;

    int lt_x1 = 72, lt_y1 = 352;          // Left target area
    int lt_x2 = 152, lt_y2 = 270;
        
    int rt_x1 = 639, rt_y1 = 353;         // Right target area
    int rt_x2 = 552, rt_y2 = 270;

    int width = 5;

    


/* Draw a box */
float box_diff(unsigned int x1, unsigned int y1, 
              unsigned int x2, unsigned int y2, 
              color_type *f1, color_type *f2)
{
    float d = 0;

    for (unsigned int y = y1; y < y2 ; y++) 
    {
        unsigned int index = x1 + y * size_x;

        // Horizontal line
        for (unsigned int x = x1; x < x2 ; x++)
        {
            d += distance(f1[index], f2[index]);
        }
    }
    
    return d;
    }
/* End of draw box */




void put_pixel(color_type *f, int x, int y)
{
f[x+y * size_x] = blue;

}


 float get_pixel_diff(color_type *f1, color_type *f2, int x, int y)
{
    int index = x+y * size_x;

    return distance(f1[index], f2[index]);
}





void draw_line(color_type * frame, int w, int x0, int y0, int  x1, int  y1 )
{
	int steep;

	if (abs(y1 - y0) > abs(x1 - x0))
		steep = 1;
	else
		steep = 0;

	int tmp;

	if (steep)
	{
		tmp = x0; x0 = y0; y0 = tmp;
		tmp = x1; x1 = y1; y1 = tmp;
	}

	if (x0 > x1)
	{
		tmp = x0; x0 = x1; x1 = tmp;
		tmp = y0; y0 = y1; y1 = tmp;
	}

	int deltax = x1 - x0;
	int deltay = abs(y1 - y0);
	int error = deltax / 2;
	int ystep;
	int y = y0;

	if (y0 < y1) 
		ystep = 1; 
	else 
		ystep = -1;

	int x;

	for (x = x0; x < x1; x++)
	{
     	if (steep)
		{
		
		for (int i = 0; i < w; i++)
    		put_pixel(frame, y + i, size_y-1-x); 
		}
		else 
		{
			for (int i = 0; i < w; i++)
        		put_pixel(frame, x+i, size_y-1-y); 
		}

       	error = error - deltay;

     	if (error < 0)
		{
           	y = y + ystep;
       		error = error + deltax;
		}
	}
}
/* End of draw_line */


float line_diff(color_type * f1, color_type * f2, int w, int x0, int y0, int  x1, int  y1 )
{
    float diff = 0;

	int steep;

	if (abs(y1 - y0) > abs(x1 - x0))
		steep = 1;
	else
		steep = 0;

	int tmp;

	if (steep)
	{
		tmp = x0; x0 = y0; y0 = tmp;
		tmp = x1; x1 = y1; y1 = tmp;
	}

	if (x0 > x1)
	{
		tmp = x0; x0 = x1; x1 = tmp;
		tmp = y0; y0 = y1; y1 = tmp;
	}

	int deltax = x1 - x0;
	int deltay = abs(y1 - y0);
	int error = deltax / 2;
	int ystep;
	int y = y0;

	if (y0 < y1) 
		ystep = 1; 
	else 
		ystep = -1;

	int x;

	for (x = x0; x < x1; x++)
	{
     	if (steep)
		{
    		for (int i = 0; i < w; i++)
                diff += get_pixel_diff(f1, f2,y + i, size_y-1-x); 
		}
		else 
		{
			for (int i = 0; i < w; i++)
    			diff += get_pixel_diff(f1, f2, x+i, size_y-1-y); 
		}

      	error = error - deltay;

       	if (error < 0)
		{
	            	y = y + ystep;
             		error = error + deltax;
		}
	}
	
	return diff;
}
/* End of draw_line */


/* Compute targetting box */
void speed_frame(color_type *s, color_type *d )
{

    float l_th = 0;
    float r_th = 0;


                    
    float lt = line_diff(s, last_frame, width, lt_x1, lt_y1, lt_x2, lt_y2);
    float rt = line_diff(s, last_frame, width,  rt_x1, rt_y1, rt_x2, rt_y2);
                      
             

    char b[300];
    
    static int mode = MODE_CLEAR;
    static double s_time = get_time();

    float speed = 0; 
        
    double dt = get_time() - s_time;

    static float ave_lt = lt;
    static float ave_rt = rt;

    static float act_lt = lt;
    static float act_rt = rt;
      
      
      
    ave_lt = ave_lt * alpha + (1 - alpha) * lt;
    ave_rt = ave_rt * alpha + (1 - alpha) * rt;

    act_lt = act_lt * act_alpha + (1 - act_alpha) * lt;
    act_rt = act_rt * act_alpha + (1 - act_alpha) * rt;

    
    static int count = 0;
    
    
    if (count++ > fps/2)
    {
        console("ave_l: %6.0f ave_r: %6.0f act_l: %6.0f act_r: %6.0f LF: %+03.2f RF: %+03.2f\n", 
                 ave_lt, ave_rt, act_lt, act_rt, (lt - ave_lt)/ave_lt, (rt-ave_rt)/ave_rt);
        count = 0;
    }

    
    
     l_th = ave_lt * fact;
     r_th = ave_rt * fact;
         
    
    // No trigger in timeout period       
    if (dt > timeout && ((mode == MODE_RIGHT) || (mode == MODE_LEFT) ))
    {
        mode = MODE_WAIT;
        sprintf(b, "Timeout\n");
        console(b);
    }

    // wait for car to clear area
    if (dt > clear && mode == MODE_CLEAR)
    {
        mode = MODE_WAIT;
        sprintf(b, "Clear Complete\n");
        console(b);  
    }   
    
    int capture = 0;
           
    if (act_lt > l_th && dt > min_time)
    {
        char b[300];
        sprintf(b, "Left: %10.0f\n", act_lt);
        console(b);
       
       if (mode == MODE_WAIT)  // inital hit on left
       {
            mode = MODE_LEFT;
            s_time = get_time();
   
       } else
       if (mode == MODE_RIGHT)   // Right to left
       {
            capture = 1;
       } 
    }


    if (act_rt > r_th && dt > min_time)
    {
        sprintf(b, "Right: %10.0f\n", act_rt);
        console(b);

       if (mode == MODE_WAIT) // inital hit on right
       {
           mode = MODE_RIGHT;
           s_time = get_time();
       } else
       if (mode == MODE_LEFT)           // right to left
       {
            capture = 1;
       } 
    }
    
    if (capture == 1)
    {
        speed = dist / dt * 0.681818182;


        int index;
  
   
        if ( mode == MODE_RIGHT)
        {
            index = (NUM_BACK_BUFFERS - 1)  - (int)(fps * dt*.20);
            sprintf(b, "R->L dt: %2.2f  spd: %4.2f mph  fps: %d index %d", dt, speed, fps, index);

        }


        if ( mode == MODE_LEFT)  
        {
            index = (NUM_BACK_BUFFERS - 1)  - (int)(fps * dt*.55);
            sprintf(b, "L->R dt: %2.2f  spd: %4.2f mph  fps: %d index %d", dt, speed, fps, index);
        }

        add_line(b);
        
        
        if (index < 0) index = 0;
        if (index > NUM_BACK_BUFFERS-1) index = NUM_BACK_BUFFERS -1;

        char ts[200]; 
        get_time_str(ts);

        char tmp[100];
        sprintf(tmp, "%s  Speed: %4.2f", ts, speed);

        char filename[300];
        sprintf(filename, "images\\%s.jpg",ts);


        save_to_disk(filename, tmp, back_buffers[index]);
        
        
        if (speed > speed_limit && 0)
        {
        
        
        
        char s_string[500];
        
        
        sprintf(s_string, "upload \"%s\"", filename);
        
        console("Sstring:  %s\n", s_string);
        
        int result = system(s_string);
        
        
        
        

    }
  
        sprintf(tmp, ",%4.2f", speed);
        addrecord(tmp);
        
        
       mode = MODE_CLEAR;



    }
            
    // Done with image, save to old image
    memcpy(last_frame, s, size_x*size_y * sizeof(color_type)); 

    // Get orig frame
    if (!d_frame)
        memcpy(d, s, sizeof(color_type)*size_x*size_y);


    
    
    draw_line(d, width, lt_x1, lt_y1, lt_x2, lt_y2);
    draw_line(d, width, rt_x1, rt_y1, rt_x2, rt_y2);
    
    

    
}
/* End of target frame */







